@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
- * Copyright  2001 Udaltsoft
+ * Copyright (C) 2001 Udaltsoft
  *
  * Written by Sergey V. Oudaltsov <svu@users.sourceforge.net>
  *
@@ -44,6 +44,8 @@
 
 #define GTK_RESPONSE_PRINT 2
 
+static GsdKeyboardManager *manager = NULL;
+
 static XklEngine *xkl_engine;
 static XklConfigRegistry *xkl_registry = NULL;
 
@@ -64,6 +66,9 @@ static void *pa_callback_user_data = NULL;
 static const char KNOWN_FILES_KEY[] =
     "/desktop/gnome/peripherals/keyboard/general/known_file_list";
 
+static const char DISABLE_INDICATOR_KEY[] =
+    "/desktop/gnome/peripherals/keyboard/general/disable_indicator";
+
 static const char *gdm_keyboard_layout = NULL;
 
 static GtkStatusIcon *icon = NULL;
@@ -140,6 +145,7 @@ apply_desktop_settings (void)
 	if (!inited_ok)
 		return;
 
+	gsd_keyboard_manager_apply_settings (manager);
 	gkbd_desktop_config_load_from_gconf (&current_config);
 	/* again, probably it would be nice to compare things
 	   before activating them */
@@ -444,6 +450,16 @@ show_hide_icon ()
 {
 	if (g_slist_length (current_kbd_config.layouts_variants) > 1) {
 		if (icon == NULL) {
+			GConfClient *conf_client =
+			    gconf_client_get_default ();
+			gboolean disable =
+			    gconf_client_get_bool (conf_client,
+						   DISABLE_INDICATOR_KEY,
+						   NULL);
+			g_object_unref (conf_client);
+			if (disable)
+				return;
+
 			xkl_debug (150, "Creating new icon\n");
 			icon = gkbd_status_new ();
 			g_signal_connect (icon, "popup-menu",
@@ -469,7 +485,6 @@ try_activating_xkb_config_if_new (GkbdKeyboardConfig *
 	if (!gkbd_keyboard_config_equals
 	    (&current_kbd_config, current_sys_kbd_config)) {
 		if (gkbd_keyboard_config_activate (&current_kbd_config)) {
-			show_hide_icon ();
 			if (pa_callback != NULL) {
 				(*pa_callback) (pa_callback_user_data);
 				return TRUE;
@@ -539,8 +554,8 @@ filter_xkb_config (void)
 					current_kbd_config.layouts_variants
 					    =
 					    g_slist_delete_link
-					    (current_kbd_config.layouts_variants,
-					     filtered);
+					    (current_kbd_config.
+					     layouts_variants, filtered);
 					any_change = TRUE;
 					continue;
 				}
@@ -588,10 +603,30 @@ apply_xkb_settings (void)
 		GSList *found_node;
 		int max_groups;
 
-		max_groups = xkl_engine_get_max_num_groups (xkl_engine);
-		layouts = gconf_client_get_list (conf_client,
-						 GKBD_KEYBOARD_CONFIG_KEY_LAYOUTS,
-						 GCONF_VALUE_STRING, NULL);
+		max_groups =
+		    MAX (xkl_engine_get_max_num_groups (xkl_engine), 1);
+		layouts =
+		    gconf_client_get_list (conf_client,
+					   GKBD_KEYBOARD_CONFIG_KEY_LAYOUTS,
+					   GCONF_VALUE_STRING, NULL);
+
+		/* Use system layouts as a default if we do not have
+		 * user configuration */
+		if (layouts == NULL) {
+			GSList *i;
+			int len;
+
+			for (i = initial_sys_kbd_config.layouts_variants;
+			     i; i = g_slist_next (i)) {
+				s = g_strdup (i->data);
+
+				/* chop off empty variants to avoid duplicates */
+				len = strlen (s);
+				if (s[len - 1] == '\t')
+					s[len - 1] = '\0';
+				layouts = g_slist_append (layouts, s);
+			}
+		}
 
 		/* Add the layout if it doesn't already exist. XKB limits the
 		 * total number of layouts. If we already have the maximum
@@ -683,6 +718,7 @@ apply_xkb_settings (void)
 		xkl_engine_lock_group (current_config.engine,
 				       group_to_activate);
 	gkbd_keyboard_config_term (&current_sys_kbd_config);
+	show_hide_icon ();
 }
 
 static void
@@ -813,7 +849,8 @@ gsd_keyboard_new_device (XklEngine * engine)
 }
 
 void
-gsd_keyboard_xkb_init (GConfClient * client)
+gsd_keyboard_xkb_init (GConfClient * client,
+		       GsdKeyboardManager * kbd_manager)
 {
 	gnome_settings_profile_start (NULL);
 #ifdef GSDKX
@@ -821,6 +858,7 @@ gsd_keyboard_xkb_init (GConfClient * client)
 	logfile = fopen ("/tmp/gsdkx.log", "a");
 	xkl_set_log_appender (gsd_keyboard_log_appender);
 #endif
+	manager = kbd_manager;
 	gnome_settings_profile_start ("xkl_engine_get_instance");
 	xkl_engine = xkl_engine_get_instance (GDK_DISPLAY ());
 	gnome_settings_profile_end ("xkl_engine_get_instance");
@@ -886,6 +924,7 @@ gsd_keyboard_xkb_shutdown (void)
 
 	pa_callback = NULL;
 	pa_callback_user_data = NULL;
+	manager = NULL;
 
 	g_hash_table_destroy (preview_dialogs);
 
